@@ -1,4 +1,5 @@
 use crate::prelude::*;
+use std::{cell::RefCell, rc::Rc};
 
 pub trait StatefulAction<T> {
     fn tick(&mut self, data: &mut T) -> Status;
@@ -17,14 +18,14 @@ pub enum Behavior<T> {
     },
     Cond(
         String,
-        // Box<dyn Fn(&mut T, &P) -> bool>,
+        // Rc<dyn Fn(&mut T, &P) -> bool>,
         fn(&T) -> bool,
-        Box<Node<T>>,
-        Box<Node<T>>,
+        Rc<RefCell<Node<T>>>,
+        Rc<RefCell<Node<T>>>,
     ),
 
-    Sequence(usize, Vec<Node<T>>),
-    Select(usize, Vec<Node<T>>),
+    Sequence(usize, Vec<Rc<RefCell<Node<T>>>>),
+    Select(usize, Vec<Rc<RefCell<Node<T>>>>),
 
     Action(String, fn(&mut T) -> Status),
     ActionSuccess(String, fn(&mut T) -> ()),
@@ -32,12 +33,12 @@ pub enum Behavior<T> {
     StatefulAction(String, Box<dyn StatefulAction<T>>),
     // StatefulAction(String, fn(&mut T, &P) -> Status),
 
-    // Invert(Box<Behavior<T>>),
-    // AlwaysSucceed(Box<Behavior<T>>),
-    // Condition(Box<dyn Fn(f64, &mut T, &P) -> bool>, Box<Behavior<T>>),
+    // Invert(Rc<Behavior<T>>),
+    // AlwaysSucceed(Rc<Behavior<T>>),
+    // Condition(Rc<dyn Fn(f64, &mut T, &P) -> bool>, Rc<Behavior<T>>),
     // WaitForever,
     // Action(T),
-    While(fn(&T) -> bool, Box<Node<T>>),
+    While(fn(&T) -> bool, Rc<RefCell<Node<T>>>),
 }
 
 fn sequence<T>(
@@ -45,7 +46,7 @@ fn sequence<T>(
     context: &mut T,
     is_sequence: bool,
     current: &mut usize,
-    xs: &mut Vec<Node<T>>,
+    xs: &mut Vec<Rc<RefCell<Node<T>>>>,
 ) -> (Status, DebugRepr) {
     let (status_positive, status_negative) = if is_sequence {
         (Status::Success, Status::Failure)
@@ -65,10 +66,10 @@ fn sequence<T>(
     }
 
     while *current < len {
-        let x = &mut xs[*current];
+        let mut x = xs[*current].borrow_mut();
 
         if x.status == Status::Success || x.status == Status::Failure {
-            x.behavior.reset();
+            x.reset();
         }
         let (res, repr) = x.tick(delta, context);
 
@@ -148,8 +149,6 @@ impl<T> Behavior<T> {
             } => {
                 *curr -= delta;
                 let status = if *curr <= 0.0 {
-                    trace!("timer reset");
-                    // *curr = *max;
                     Status::Success
                 } else {
                     Status::Running
@@ -162,9 +161,9 @@ impl<T> Behavior<T> {
                 let c = cond(context);
 
                 let (status, child_repr) = if c {
-                    a.tick(delta, context)
+                    a.borrow_mut().tick(delta, context)
                 } else {
-                    b.tick(delta, context)
+                    b.borrow_mut().tick(delta, context)
                 };
 
                 // let mut repr = DebugRepr::new("If", Cursor::Condition(c), status);
@@ -208,7 +207,7 @@ impl<T> Behavior<T> {
 
             Behavior::While(cond, behavior) => {
                 if cond(context) {
-                    return behavior.tick(delta, context);
+                    return behavior.borrow_mut().tick(delta, context);
                 } else {
                     let status = Status::Failure;
                     return (status, DebugRepr::new("while", Cursor::Leaf, status));
@@ -257,8 +256,17 @@ impl<T> Behavior<T> {
             Behavior::Wait { ref mut curr, max } => {
                 *curr = *max;
             }
-            Behavior::Sequence(ref mut idx, _) => {
+            Behavior::Sequence(ref mut idx, nodes) => {
                 *idx = 0;
+                for node in nodes.iter_mut() {
+                    node.borrow_mut().reset();
+                }
+            }
+            Behavior::Select(ref mut idx, nodes) => {
+                *idx = 0;
+                for node in nodes.iter_mut() {
+                    node.borrow_mut().reset();
+                }
             }
             Behavior::StatefulAction(_name, ref mut state) => {
                 state.reset();
