@@ -1,4 +1,5 @@
 use crate::prelude::*;
+use std::{cell::RefCell, rc::Rc};
 
 pub struct Node<T> {
     pub name: Option<String>,
@@ -12,7 +13,7 @@ impl<T> Node<T> {
         Node {
             name: None,
             behavior,
-            status: Status::Running,
+            status: Status::Initialized,
             collapse_as: None,
         }
     }
@@ -21,7 +22,7 @@ impl<T> Node<T> {
         Node {
             name: Some(name),
             behavior,
-            status: Status::Running,
+            status: Status::Initialized,
             collapse_as: None,
         }
     }
@@ -45,25 +46,60 @@ impl<T> Node<T> {
     }
 
     pub fn sequence(nodes: Vec<Node<T>>) -> Node<T> {
-        Self::new(Behavior::Sequence(0, nodes))
+        Self::new(Behavior::Sequence(
+            0,
+            nodes
+                .into_iter()
+                .map(|node| Rc::new(RefCell::new(node)))
+                .collect(),
+        ))
     }
 
     pub fn named_sequence(name: &str, nodes: Vec<Node<T>>) -> Node<T> {
-        Self::new_named(name.to_owned(), Behavior::Sequence(0, nodes))
+        Self::new_named(
+            name.to_owned(),
+            Behavior::Sequence(
+                0,
+                nodes
+                    .into_iter()
+                    .map(|node| Rc::new(RefCell::new(node)))
+                    .collect(),
+            ),
+        )
     }
 
     pub fn select(nodes: Vec<Node<T>>) -> Node<T> {
-        Self::new(Behavior::Select(0, nodes))
+        Self::new(Behavior::Select(
+            0,
+            nodes
+                .into_iter()
+                .map(|node| Rc::new(RefCell::new(node)))
+                .collect(),
+        ))
     }
 
     pub fn named_select(name: &str, nodes: Vec<Node<T>>) -> Node<T> {
-        Self::new_named(name.to_owned(), Behavior::Select(0, nodes))
+        Self::new_named(
+            name.to_owned(),
+            Behavior::Select(
+                0,
+                nodes
+                    .into_iter()
+                    .map(|node| Rc::new(RefCell::new(node)))
+                    .collect(),
+            ),
+        )
     }
 
     pub fn cond(name: &str, cond: fn(&T) -> bool, success: Node<T>, failure: Node<T>) -> Node<T> {
         Self::new_named(
             name.to_owned(),
-            Behavior::Cond(name.to_owned(), cond, Box::new(success), Box::new(failure)),
+            Behavior::Cond(
+                name.to_owned(),
+                cond,
+                Rc::new(RefCell::new(success)),
+                Rc::new(RefCell::new(failure)),
+            ),
         )
     }
 
@@ -75,23 +111,33 @@ impl<T> Node<T> {
     }
 
     pub fn named_while_single(name: &str, cond: fn(&T) -> bool, child: Node<T>) -> Node<T> {
-        Self::new_named(name.to_owned(), Behavior::While(cond, Box::new(child)))
+        Self::new_named(
+            name.to_owned(),
+            Behavior::While(cond, Rc::new(RefCell::new(child))),
+        )
     }
 
     pub fn while_single(cond: fn(&T) -> bool, child: Node<T>) -> Node<T> {
-        Self::new(Behavior::While(cond, Box::new(child)))
+        Self::new(Behavior::While(cond, Rc::new(RefCell::new(child))))
     }
 
-    pub fn collapse(self, desc: &str) -> Node<T> {
-        Self {
-            collapse_as: Some(desc.to_owned()),
-            ..self
-        }
+    pub fn collapse(self, _desc: &str) -> Node<T> {
+        self
+        // TODO: re-enable once behavior-tree-egui catches up
+        // Self {
+        //     collapse_as: Some(desc.to_owned()),
+        //     ..self
+        // }
+    }
+
+    pub fn reset(&mut self) {
+        self.status = Status::Initialized;
+        self.behavior.reset();
     }
 
     pub fn tick(&mut self, delta: f64, context: &mut T) -> (Status, DebugRepr) {
         if self.status == Status::Success || self.status == Status::Failure {
-            self.behavior.reset();
+            self.reset();
         }
 
         let (status, repr) = self.behavior.tick(delta, context);
@@ -99,15 +145,29 @@ impl<T> Node<T> {
         (status, repr)
     }
 
+    pub fn children(&self) -> Vec<Rc<RefCell<Node<T>>>> {
+        match &self.behavior {
+            Behavior::Wait { .. } => vec![],
+            Behavior::Cond(_, _, positive, negative) => vec![positive.clone(), negative.clone()],
+            _ => vec![],
+            Behavior::Sequence(_, ref seq) => seq.clone(),
+            Behavior::Select(_, _) => todo!(),
+            Behavior::Action(_, _) => todo!(),
+            Behavior::ActionSuccess(_, _) => todo!(),
+            Behavior::StatefulAction(_, _) => todo!(),
+            Behavior::While(_, _) => todo!(),
+        }
+    }
+
     pub fn to_debug(&self) -> TreeRepr {
-        match &self.collapse_as {
+        let mut repr = match &self.collapse_as {
             Some(collapse_text) => TreeRepr::new(collapse_text, vec![]),
             None => {
                 match &self.behavior {
                     Behavior::Wait { curr, max } => TreeRepr::new("Wait", vec![])
                         .with_detail(format!("curr={}, max={}", curr, max)),
                     Behavior::Cond(name, _cond, a, b) => {
-                        TreeRepr::new("Cond", vec![a.to_debug(), b.to_debug()])
+                        TreeRepr::new("Cond", vec![a.borrow().to_debug(), b.borrow().to_debug()])
                             .with_detail(name.clone())
                     }
                     Behavior::Sequence(_, seq) => TreeRepr::new(
@@ -116,7 +176,7 @@ impl<T> Node<T> {
                         } else {
                             "Sequence".to_string()
                         },
-                        seq.iter().map(|x| x.to_debug()).collect(),
+                        seq.iter().map(|x| x.borrow().to_debug()).collect(),
                     ),
                     Behavior::Select(_, seq) => TreeRepr::new(
                         if let Some(ref name) = self.name {
@@ -124,7 +184,7 @@ impl<T> Node<T> {
                         } else {
                             "Select".to_string()
                         },
-                        seq.iter().map(|x| x.to_debug()).collect(),
+                        seq.iter().map(|x| x.borrow().to_debug()).collect(),
                     ),
                     Behavior::Action(name, _) => {
                         TreeRepr::new("Action", vec![]).with_detail(name.clone())
@@ -137,9 +197,12 @@ impl<T> Node<T> {
                     }
                     // Behavior::While(_, x) => TreeRepr::new("While", vec![x.to_debug()]),
                     // TODO: add to detail
-                    Behavior::While(_, x) => x.to_debug(),
+                    Behavior::While(_, x) => x.borrow().to_debug(),
                 }
             }
-        }
+        };
+
+        repr.status = self.status;
+        repr
     }
 }
